/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __MS_DBUS_SYSTEMD_SYSUPDATED_H__
#define __MS_DBUS_SYSTEMD_SYSUPDATED_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.sysupdate1.Manager */

#define MS_DBUS_SYSUPDATE_TYPE_MANAGER (ms_dbus_sysupdate_manager_get_type ())
#define MS_DBUS_SYSUPDATE_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER, MsDBusSysupdateManager))
#define MS_DBUS_SYSUPDATE_IS_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER))
#define MS_DBUS_SYSUPDATE_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER, MsDBusSysupdateManagerIface))

struct _MsDBusSysupdateManager;
typedef struct _MsDBusSysupdateManager MsDBusSysupdateManager;
typedef struct _MsDBusSysupdateManagerIface MsDBusSysupdateManagerIface;

struct _MsDBusSysupdateManagerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_list_app_stream) (
    MsDBusSysupdateManager *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_list_jobs) (
    MsDBusSysupdateManager *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_list_targets) (
    MsDBusSysupdateManager *object,
    GDBusMethodInvocation *invocation);

  void (*job_removed) (
    MsDBusSysupdateManager *object,
    guint64 arg_id,
    const gchar *arg_path,
    gint arg_status);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateManager, g_object_unref)
#endif

GType ms_dbus_sysupdate_manager_get_type (void);

GDBusInterfaceInfo *ms_dbus_sysupdate_manager_interface_info (void);
guint ms_dbus_sysupdate_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ms_dbus_sysupdate_manager_complete_list_targets (
    MsDBusSysupdateManager *object,
    GDBusMethodInvocation *invocation,
    GVariant *targets);

void ms_dbus_sysupdate_manager_complete_list_jobs (
    MsDBusSysupdateManager *object,
    GDBusMethodInvocation *invocation,
    GVariant *jobs);

void ms_dbus_sysupdate_manager_complete_list_app_stream (
    MsDBusSysupdateManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *urls);



/* D-Bus signal emissions functions: */
void ms_dbus_sysupdate_manager_emit_job_removed (
    MsDBusSysupdateManager *object,
    guint64 arg_id,
    const gchar *arg_path,
    gint arg_status);



/* D-Bus method calls: */
void ms_dbus_sysupdate_manager_call_list_targets (
    MsDBusSysupdateManager *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_manager_call_list_targets_finish (
    MsDBusSysupdateManager *proxy,
    GVariant ** out_targets,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_manager_call_list_targets_sync (
    MsDBusSysupdateManager *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GVariant ** out_targets,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_manager_call_list_jobs (
    MsDBusSysupdateManager *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_manager_call_list_jobs_finish (
    MsDBusSysupdateManager *proxy,
    GVariant ** out_jobs,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_manager_call_list_jobs_sync (
    MsDBusSysupdateManager *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GVariant ** out_jobs,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_manager_call_list_app_stream (
    MsDBusSysupdateManager *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_manager_call_list_app_stream_finish (
    MsDBusSysupdateManager *proxy,
    gchar *** out_urls,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_manager_call_list_app_stream_sync (
    MsDBusSysupdateManager *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar *** out_urls,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define MS_DBUS_SYSUPDATE_TYPE_MANAGER_PROXY (ms_dbus_sysupdate_manager_proxy_get_type ())
#define MS_DBUS_SYSUPDATE_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER_PROXY, MsDBusSysupdateManagerProxy))
#define MS_DBUS_SYSUPDATE_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_SYSUPDATE_TYPE_MANAGER_PROXY, MsDBusSysupdateManagerProxyClass))
#define MS_DBUS_SYSUPDATE_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER_PROXY, MsDBusSysupdateManagerProxyClass))
#define MS_DBUS_SYSUPDATE_IS_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER_PROXY))
#define MS_DBUS_SYSUPDATE_IS_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_SYSUPDATE_TYPE_MANAGER_PROXY))

typedef struct _MsDBusSysupdateManagerProxy MsDBusSysupdateManagerProxy;
typedef struct _MsDBusSysupdateManagerProxyClass MsDBusSysupdateManagerProxyClass;
typedef struct _MsDBusSysupdateManagerProxyPrivate MsDBusSysupdateManagerProxyPrivate;

struct _MsDBusSysupdateManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MsDBusSysupdateManagerProxyPrivate *priv;
};

struct _MsDBusSysupdateManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType ms_dbus_sysupdate_manager_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateManagerProxy, g_object_unref)
#endif

void ms_dbus_sysupdate_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSysupdateManager *ms_dbus_sysupdate_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSysupdateManager *ms_dbus_sysupdate_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ms_dbus_sysupdate_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSysupdateManager *ms_dbus_sysupdate_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSysupdateManager *ms_dbus_sysupdate_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define MS_DBUS_SYSUPDATE_TYPE_MANAGER_SKELETON (ms_dbus_sysupdate_manager_skeleton_get_type ())
#define MS_DBUS_SYSUPDATE_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER_SKELETON, MsDBusSysupdateManagerSkeleton))
#define MS_DBUS_SYSUPDATE_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_SYSUPDATE_TYPE_MANAGER_SKELETON, MsDBusSysupdateManagerSkeletonClass))
#define MS_DBUS_SYSUPDATE_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER_SKELETON, MsDBusSysupdateManagerSkeletonClass))
#define MS_DBUS_SYSUPDATE_IS_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_MANAGER_SKELETON))
#define MS_DBUS_SYSUPDATE_IS_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_SYSUPDATE_TYPE_MANAGER_SKELETON))

typedef struct _MsDBusSysupdateManagerSkeleton MsDBusSysupdateManagerSkeleton;
typedef struct _MsDBusSysupdateManagerSkeletonClass MsDBusSysupdateManagerSkeletonClass;
typedef struct _MsDBusSysupdateManagerSkeletonPrivate MsDBusSysupdateManagerSkeletonPrivate;

struct _MsDBusSysupdateManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MsDBusSysupdateManagerSkeletonPrivate *priv;
};

struct _MsDBusSysupdateManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ms_dbus_sysupdate_manager_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateManagerSkeleton, g_object_unref)
#endif

MsDBusSysupdateManager *ms_dbus_sysupdate_manager_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.sysupdate1.Job */

#define MS_DBUS_SYSUPDATE_TYPE_JOB (ms_dbus_sysupdate_job_get_type ())
#define MS_DBUS_SYSUPDATE_JOB(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_JOB, MsDBusSysupdateJob))
#define MS_DBUS_SYSUPDATE_IS_JOB(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_JOB))
#define MS_DBUS_SYSUPDATE_JOB_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), MS_DBUS_SYSUPDATE_TYPE_JOB, MsDBusSysupdateJobIface))

struct _MsDBusSysupdateJob;
typedef struct _MsDBusSysupdateJob MsDBusSysupdateJob;
typedef struct _MsDBusSysupdateJobIface MsDBusSysupdateJobIface;

struct _MsDBusSysupdateJobIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_cancel) (
    MsDBusSysupdateJob *object,
    GDBusMethodInvocation *invocation);

  guint64  (*get_id) (MsDBusSysupdateJob *object);

  gboolean  (*get_offline) (MsDBusSysupdateJob *object);

  guint  (*get_progress) (MsDBusSysupdateJob *object);

  const gchar * (*get_type_) (MsDBusSysupdateJob *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateJob, g_object_unref)
#endif

GType ms_dbus_sysupdate_job_get_type (void);

GDBusInterfaceInfo *ms_dbus_sysupdate_job_interface_info (void);
guint ms_dbus_sysupdate_job_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ms_dbus_sysupdate_job_complete_cancel (
    MsDBusSysupdateJob *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void ms_dbus_sysupdate_job_call_cancel (
    MsDBusSysupdateJob *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_job_call_cancel_finish (
    MsDBusSysupdateJob *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_job_call_cancel_sync (
    MsDBusSysupdateJob *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
guint64 ms_dbus_sysupdate_job_get_id (MsDBusSysupdateJob *object);
void ms_dbus_sysupdate_job_set_id (MsDBusSysupdateJob *object, guint64 value);

const gchar *ms_dbus_sysupdate_job_get_type_ (MsDBusSysupdateJob *object);
gchar *ms_dbus_sysupdate_job_dup_type_ (MsDBusSysupdateJob *object);
void ms_dbus_sysupdate_job_set_type_ (MsDBusSysupdateJob *object, const gchar *value);

gboolean ms_dbus_sysupdate_job_get_offline (MsDBusSysupdateJob *object);
void ms_dbus_sysupdate_job_set_offline (MsDBusSysupdateJob *object, gboolean value);

guint ms_dbus_sysupdate_job_get_progress (MsDBusSysupdateJob *object);
void ms_dbus_sysupdate_job_set_progress (MsDBusSysupdateJob *object, guint value);


/* ---- */

#define MS_DBUS_SYSUPDATE_TYPE_JOB_PROXY (ms_dbus_sysupdate_job_proxy_get_type ())
#define MS_DBUS_SYSUPDATE_JOB_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_JOB_PROXY, MsDBusSysupdateJobProxy))
#define MS_DBUS_SYSUPDATE_JOB_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_SYSUPDATE_TYPE_JOB_PROXY, MsDBusSysupdateJobProxyClass))
#define MS_DBUS_SYSUPDATE_JOB_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_SYSUPDATE_TYPE_JOB_PROXY, MsDBusSysupdateJobProxyClass))
#define MS_DBUS_SYSUPDATE_IS_JOB_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_JOB_PROXY))
#define MS_DBUS_SYSUPDATE_IS_JOB_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_SYSUPDATE_TYPE_JOB_PROXY))

typedef struct _MsDBusSysupdateJobProxy MsDBusSysupdateJobProxy;
typedef struct _MsDBusSysupdateJobProxyClass MsDBusSysupdateJobProxyClass;
typedef struct _MsDBusSysupdateJobProxyPrivate MsDBusSysupdateJobProxyPrivate;

struct _MsDBusSysupdateJobProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MsDBusSysupdateJobProxyPrivate *priv;
};

struct _MsDBusSysupdateJobProxyClass
{
  GDBusProxyClass parent_class;
};

GType ms_dbus_sysupdate_job_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateJobProxy, g_object_unref)
#endif

void ms_dbus_sysupdate_job_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSysupdateJob *ms_dbus_sysupdate_job_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSysupdateJob *ms_dbus_sysupdate_job_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ms_dbus_sysupdate_job_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSysupdateJob *ms_dbus_sysupdate_job_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSysupdateJob *ms_dbus_sysupdate_job_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define MS_DBUS_SYSUPDATE_TYPE_JOB_SKELETON (ms_dbus_sysupdate_job_skeleton_get_type ())
#define MS_DBUS_SYSUPDATE_JOB_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_JOB_SKELETON, MsDBusSysupdateJobSkeleton))
#define MS_DBUS_SYSUPDATE_JOB_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_SYSUPDATE_TYPE_JOB_SKELETON, MsDBusSysupdateJobSkeletonClass))
#define MS_DBUS_SYSUPDATE_JOB_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_SYSUPDATE_TYPE_JOB_SKELETON, MsDBusSysupdateJobSkeletonClass))
#define MS_DBUS_SYSUPDATE_IS_JOB_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_JOB_SKELETON))
#define MS_DBUS_SYSUPDATE_IS_JOB_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_SYSUPDATE_TYPE_JOB_SKELETON))

typedef struct _MsDBusSysupdateJobSkeleton MsDBusSysupdateJobSkeleton;
typedef struct _MsDBusSysupdateJobSkeletonClass MsDBusSysupdateJobSkeletonClass;
typedef struct _MsDBusSysupdateJobSkeletonPrivate MsDBusSysupdateJobSkeletonPrivate;

struct _MsDBusSysupdateJobSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MsDBusSysupdateJobSkeletonPrivate *priv;
};

struct _MsDBusSysupdateJobSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ms_dbus_sysupdate_job_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateJobSkeleton, g_object_unref)
#endif

MsDBusSysupdateJob *ms_dbus_sysupdate_job_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.sysupdate1.Target */

#define MS_DBUS_SYSUPDATE_TYPE_TARGET (ms_dbus_sysupdate_target_get_type ())
#define MS_DBUS_SYSUPDATE_TARGET(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET, MsDBusSysupdateTarget))
#define MS_DBUS_SYSUPDATE_IS_TARGET(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET))
#define MS_DBUS_SYSUPDATE_TARGET_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET, MsDBusSysupdateTargetIface))

struct _MsDBusSysupdateTarget;
typedef struct _MsDBusSysupdateTarget MsDBusSysupdateTarget;
typedef struct _MsDBusSysupdateTargetIface MsDBusSysupdateTargetIface;

struct _MsDBusSysupdateTargetIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_acquire) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_new_version,
    guint64 arg_flags);

  gboolean (*handle_check_new) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_describe) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_version,
    guint64 arg_flags);

  gboolean (*handle_describe_feature) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_feature,
    guint64 arg_flags);

  gboolean (*handle_get_app_stream) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_version) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_install) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_new_version,
    guint64 arg_flags);

  gboolean (*handle_list) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    guint64 arg_flags);

  gboolean (*handle_list_features) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    guint64 arg_flags);

  gboolean (*handle_set_feature_enabled) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_feature,
    gint arg_enabled,
    guint64 arg_flags);

  gboolean (*handle_vacuum) (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation);

  const gchar * (*get_class) (MsDBusSysupdateTarget *object);

  const gchar * (*get_name) (MsDBusSysupdateTarget *object);

  const gchar * (*get_path) (MsDBusSysupdateTarget *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateTarget, g_object_unref)
#endif

GType ms_dbus_sysupdate_target_get_type (void);

GDBusInterfaceInfo *ms_dbus_sysupdate_target_interface_info (void);
guint ms_dbus_sysupdate_target_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ms_dbus_sysupdate_target_complete_list (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *versions);

void ms_dbus_sysupdate_target_complete_describe (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *json);

void ms_dbus_sysupdate_target_complete_check_new (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *new_version);

void ms_dbus_sysupdate_target_complete_acquire (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *new_version,
    guint64 job_id,
    const gchar *job_path);

void ms_dbus_sysupdate_target_complete_install (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *new_version,
    guint64 job_id,
    const gchar *job_path);

void ms_dbus_sysupdate_target_complete_vacuum (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    guint instances,
    guint disabled_transfers);

void ms_dbus_sysupdate_target_complete_get_app_stream (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *appstream);

void ms_dbus_sysupdate_target_complete_get_version (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *version);

void ms_dbus_sysupdate_target_complete_list_features (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *features);

void ms_dbus_sysupdate_target_complete_describe_feature (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation,
    const gchar *json);

void ms_dbus_sysupdate_target_complete_set_feature_enabled (
    MsDBusSysupdateTarget *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void ms_dbus_sysupdate_target_call_list (
    MsDBusSysupdateTarget *proxy,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_list_finish (
    MsDBusSysupdateTarget *proxy,
    gchar *** out_versions,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_list_sync (
    MsDBusSysupdateTarget *proxy,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar *** out_versions,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_describe (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_version,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_describe_finish (
    MsDBusSysupdateTarget *proxy,
    gchar ** out_json,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_describe_sync (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_version,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar ** out_json,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_check_new (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_check_new_finish (
    MsDBusSysupdateTarget *proxy,
    gchar ** out_new_version,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_check_new_sync (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar ** out_new_version,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_acquire (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_new_version,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_acquire_finish (
    MsDBusSysupdateTarget *proxy,
    gchar ** out_new_version,
    guint64* out_job_id,
    gchar ** out_job_path,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_acquire_sync (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_new_version,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar ** out_new_version,
    guint64* out_job_id,
    gchar ** out_job_path,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_install (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_new_version,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_install_finish (
    MsDBusSysupdateTarget *proxy,
    gchar ** out_new_version,
    guint64* out_job_id,
    gchar ** out_job_path,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_install_sync (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_new_version,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar ** out_new_version,
    guint64* out_job_id,
    gchar ** out_job_path,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_vacuum (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_vacuum_finish (
    MsDBusSysupdateTarget *proxy,
    guint* out_instances,
    guint* out_disabled_transfers,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_vacuum_sync (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    guint* out_instances,
    guint* out_disabled_transfers,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_get_app_stream (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_get_app_stream_finish (
    MsDBusSysupdateTarget *proxy,
    gchar *** out_appstream,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_get_app_stream_sync (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar *** out_appstream,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_get_version (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_get_version_finish (
    MsDBusSysupdateTarget *proxy,
    gchar ** out_version,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_get_version_sync (
    MsDBusSysupdateTarget *proxy,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar ** out_version,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_list_features (
    MsDBusSysupdateTarget *proxy,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_list_features_finish (
    MsDBusSysupdateTarget *proxy,
    gchar *** out_features,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_list_features_sync (
    MsDBusSysupdateTarget *proxy,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar *** out_features,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_describe_feature (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_feature,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_describe_feature_finish (
    MsDBusSysupdateTarget *proxy,
    gchar ** out_json,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_describe_feature_sync (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_feature,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    gchar ** out_json,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sysupdate_target_call_set_feature_enabled (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_feature,
    gint arg_enabled,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sysupdate_target_call_set_feature_enabled_finish (
    MsDBusSysupdateTarget *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sysupdate_target_call_set_feature_enabled_sync (
    MsDBusSysupdateTarget *proxy,
    const gchar *arg_feature,
    gint arg_enabled,
    guint64 arg_flags,
    GDBusCallFlags call_flags,
    gint timeout_msec,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *ms_dbus_sysupdate_target_get_class (MsDBusSysupdateTarget *object);
gchar *ms_dbus_sysupdate_target_dup_class (MsDBusSysupdateTarget *object);
void ms_dbus_sysupdate_target_set_class (MsDBusSysupdateTarget *object, const gchar *value);

const gchar *ms_dbus_sysupdate_target_get_name (MsDBusSysupdateTarget *object);
gchar *ms_dbus_sysupdate_target_dup_name (MsDBusSysupdateTarget *object);
void ms_dbus_sysupdate_target_set_name (MsDBusSysupdateTarget *object, const gchar *value);

const gchar *ms_dbus_sysupdate_target_get_path (MsDBusSysupdateTarget *object);
gchar *ms_dbus_sysupdate_target_dup_path (MsDBusSysupdateTarget *object);
void ms_dbus_sysupdate_target_set_path (MsDBusSysupdateTarget *object, const gchar *value);


/* ---- */

#define MS_DBUS_SYSUPDATE_TYPE_TARGET_PROXY (ms_dbus_sysupdate_target_proxy_get_type ())
#define MS_DBUS_SYSUPDATE_TARGET_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET_PROXY, MsDBusSysupdateTargetProxy))
#define MS_DBUS_SYSUPDATE_TARGET_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_SYSUPDATE_TYPE_TARGET_PROXY, MsDBusSysupdateTargetProxyClass))
#define MS_DBUS_SYSUPDATE_TARGET_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET_PROXY, MsDBusSysupdateTargetProxyClass))
#define MS_DBUS_SYSUPDATE_IS_TARGET_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET_PROXY))
#define MS_DBUS_SYSUPDATE_IS_TARGET_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_SYSUPDATE_TYPE_TARGET_PROXY))

typedef struct _MsDBusSysupdateTargetProxy MsDBusSysupdateTargetProxy;
typedef struct _MsDBusSysupdateTargetProxyClass MsDBusSysupdateTargetProxyClass;
typedef struct _MsDBusSysupdateTargetProxyPrivate MsDBusSysupdateTargetProxyPrivate;

struct _MsDBusSysupdateTargetProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MsDBusSysupdateTargetProxyPrivate *priv;
};

struct _MsDBusSysupdateTargetProxyClass
{
  GDBusProxyClass parent_class;
};

GType ms_dbus_sysupdate_target_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateTargetProxy, g_object_unref)
#endif

void ms_dbus_sysupdate_target_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSysupdateTarget *ms_dbus_sysupdate_target_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSysupdateTarget *ms_dbus_sysupdate_target_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ms_dbus_sysupdate_target_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSysupdateTarget *ms_dbus_sysupdate_target_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSysupdateTarget *ms_dbus_sysupdate_target_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define MS_DBUS_SYSUPDATE_TYPE_TARGET_SKELETON (ms_dbus_sysupdate_target_skeleton_get_type ())
#define MS_DBUS_SYSUPDATE_TARGET_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET_SKELETON, MsDBusSysupdateTargetSkeleton))
#define MS_DBUS_SYSUPDATE_TARGET_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_SYSUPDATE_TYPE_TARGET_SKELETON, MsDBusSysupdateTargetSkeletonClass))
#define MS_DBUS_SYSUPDATE_TARGET_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET_SKELETON, MsDBusSysupdateTargetSkeletonClass))
#define MS_DBUS_SYSUPDATE_IS_TARGET_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_SYSUPDATE_TYPE_TARGET_SKELETON))
#define MS_DBUS_SYSUPDATE_IS_TARGET_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_SYSUPDATE_TYPE_TARGET_SKELETON))

typedef struct _MsDBusSysupdateTargetSkeleton MsDBusSysupdateTargetSkeleton;
typedef struct _MsDBusSysupdateTargetSkeletonClass MsDBusSysupdateTargetSkeletonClass;
typedef struct _MsDBusSysupdateTargetSkeletonPrivate MsDBusSysupdateTargetSkeletonPrivate;

struct _MsDBusSysupdateTargetSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MsDBusSysupdateTargetSkeletonPrivate *priv;
};

struct _MsDBusSysupdateTargetSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ms_dbus_sysupdate_target_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSysupdateTargetSkeleton, g_object_unref)
#endif

MsDBusSysupdateTarget *ms_dbus_sysupdate_target_skeleton_new (void);


G_END_DECLS

#endif /* __MS_DBUS_SYSTEMD_SYSUPDATED_H__ */
