
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

            #pragma once

            #include <glib-object.h>
            #if !defined(_LIBPMS_INSIDE) && !defined(LIBPMS_COMPILATION)
#error "Only <libpms.h> can be included directly."
#endif

#include "libpms-version.h"

            G_BEGIN_DECLS

/* enumerations from "libpms-enums.h" */

PMS_AVAILABLE_IN_ALL
GType ms_os_update_state_get_type (void);
#define MS_TYPE_OS_UPDATE_STATE (ms_os_update_state_get_type())

G_END_DECLS

/* Generated data ends here */

