
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "mobile-settings-config.h"
#include "libpms-enum-types.h"
#include "libpms-enums.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "libpms-enums.h" */

GType
ms_os_update_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (MS_OS_UPDATE_STATE_FAILED), "MS_OS_UPDATE_STATE_FAILED", "failed" },
        { C_ENUM (MS_OS_UPDATE_STATE_UNKNOWN), "MS_OS_UPDATE_STATE_UNKNOWN", "unknown" },
        { C_ENUM (MS_OS_UPDATE_STATE_READY), "MS_OS_UPDATE_STATE_READY", "ready" },
        { C_ENUM (MS_OS_UPDATE_STATE_FETCHING), "MS_OS_UPDATE_STATE_FETCHING", "fetching" },
        { C_ENUM (MS_OS_UPDATE_STATE_FETCHED), "MS_OS_UPDATE_STATE_FETCHED", "fetched" },
        { C_ENUM (MS_OS_UPDATE_STATE_INSTALLING), "MS_OS_UPDATE_STATE_INSTALLING", "installing" },
        { C_ENUM (MS_OS_UPDATE_STATE_INSTALLED), "MS_OS_UPDATE_STATE_INSTALLED", "installed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("MsOsUpdateState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

